#	n1---------n2
#
#	Simulacin de una cola M/D/1
# 
#	Se determinan los siguientes parmetros del sistema:
#
#	Ro -	Utilizacin del servidor. 
#	N  - 	Nmero promedio de paquetes en la cola.

set ns [new Simulator]
set f [open salida.tr w]
$ns trace-all $f
set fin_sim 1000

set lamda 8.0
set tarribo [expr 1/$lamda] 
set cenlace 80000
set tampa 500
set mu [expr $cenlace/(8*$tampa)]

# crea un archivo para registrar el tamao de la cola
set tq [open cola.q w]


#
# Se genera la topologa de la red
#

# crea los nodos
set n1 [$ns node]
set n2 [$ns node]


# crea el enlace entre los nodos n1 y n2
$ns simplex-link $n1 $n2 $cenlace 0ms DropTail

      

# crea un agente UDP y lo asigna al nodo n1
set fuente [new Agent/UDP]
$ns attach-agent $n1 $fuente


# crea un receptor y lo asigna a n2.
set sink [new Agent/Null] 
$ns attach-agent $n2 $sink

# conecta la fuente y el receptor

$ns connect $fuente $sink

# crea un generador de trfico Poisson y lo asigna al nodo UDP

set tgen [new Application/Traffic/Exponential]
$tgen attach-agent $fuente

# configura los parmetros del generador de trfico tal que el mismo 
# genera un paquete cada cierto tiempo t, donde t es una V.A. con
# distribucin exponencial de media 1/lamda .

$tgen set packetSize_ $tampa
$tgen set burst_time_ 0ms
$tgen set idle_time_ $tarribo 
$tgen set rate_ 100Mb

# NOTA: El generador de trafico arriba configurado es un generador exponencial de tipo on/off, donde 
# burst_time e idle_time son respectivamente los tiempos durante el cual la fuente est activa e inactiva,
# siendo dichos tiempos V.A. con distribucin exponencial. 

# Para generar un proceso Poisson de generacin de paquetes seteamos el valor de burst_time en 0 (el NS 
# asegura que en este caso se enviar un solo paquete) y la tasa de generacin en un valor grande (de tal 
# forma de que el tiempo entre paquetes quede determinado puramente por el idle_time).


# crea un monitor de cola
$ns trace-queue $n1 $n2
set qmon [$ns monitor-queue $n1 $n2 ""]


# se establece el tamao mximo de la cola en paquetes
$ns queue-limit $n1 $n2 200000


# Procedimiento que se llama periodicamente (cada time seg) para registrar el tamao de la cola.


       
proc record {} {
	global ns qmon tq       
	set now [$ns now]
	set time 0.01
	set tamcol [eval $qmon set pkts_]
	puts $tq "$now $tamcol "
	$ns at [expr $now+$time] "record"
}


# Procedimiento de finalizacin

proc finish {} {
      global ns qmon fin_sim lamda mu 
     

$ns flush-trace

puts ""              
puts "Fin de la simulacin.  Tiempo de simulacin:  $fin_sim s"      
puts ""              
puts ""    
puts [format "Lambda = %5.5f" $lamda]
puts [format "Mu = %5.5f" $mu]
puts ""

set parri [eval $qmon set parrivals_]      
set barri [eval $qmon set barrivals_]
set paten [eval $qmon set pdepartures_]      
set baten [eval $qmon set bdepartures_]   
puts ""

# se calcula la utilizacin del sistema de dos formas
# (i)   cociente entre lamda y mu   
# (ii)  cociente entre tiempo que el servidor est ocupado y tiempo total de simulacin

set ro1 [expr 1.0*$lamda/$mu]
set busy [expr 1.0*$paten/$mu]
set ro2 [expr 1.0*$busy/$fin_sim]

puts [format "Utilizacin del sistema (lambda/mu) = %9.3f" $ro1]
puts ""
puts [format "Utilizacin del sistema (T_busy/T_sim) = %9.3f" $ro2]
puts ""

set pktsInt [$qmon get-pkts-integrator]
set suma [expr [$pktsInt set sum_]]
set avg_queue [expr $suma/$fin_sim]
puts [format "N promedio de paquetes en la cola = %9.5f paquetes" $avg_queue]
puts ""


exec xgraph cola.q -geometry 800x400 &

exit 0

}

# se indican las operaciones

$ns at 0 "record"
$ns at 0.01 "$tgen start"
$ns at $fin_sim "$tgen stop"
$ns at [expr $fin_sim+0.01] "finish"

$ns run


