#	n1---------n2
#
#	Simulacin de una cola M/M/1
# 
#	Se determinan los siguientes parmetros del sistema:
#
#	Ro -	Utilizacin del servidor. 
#	N  - 	Nmero promedio de paquetes en la cola.


set ns [new Simulator]

set tf [open out.tr w] 
$ns trace-all $tf

# Traza par el nam
# set nf [open out.nam w]
# $ns namtrace-all $nf

set lamda 50.0
set mu 80.0
set fin_sim 1000

set n1 [$ns node]
set n2 [$ns node]

# Como el tamao de los paquetes ser generado aleatoriamente, debemos redondear su valor
# a un entero. Para tener pequeos errores en el redondeo, tomaremos tamaos de paquete 
# grandes y, en consecuencia, gran ancho de banda para el enlace.

set link [$ns simplex-link $n1 $n2 100kb 0ms DropTail]

# Limitamos el tamao de la cola

$ns queue-limit $n1 $n2 100000

# Generamos aleatoriamente tiempos de arribo de paquetes y tamaos de paquetes de acuerdo
# a una distribucin exponencial.

set InterArrivalTime [new RandomVariable/Exponential]
$InterArrivalTime set avg_ [expr 1/$lamda]
set pktSize [new RandomVariable/Exponential]
# tamao medio de paquete (Bytes)
$pktSize set avg_ [expr 100000.0/(8*$mu)]

set src [new Agent/UDP]
$ns attach-agent $n1 $src

set sink [new Agent/Null]
$ns attach-agent $n2 $sink
$ns connect $src $sink

# Creamos un monitor de cola

set tq [open cola.q w]
set qmon [$ns monitor-queue $n1 $n2 ""]


proc finish {} {
	global ns tf tq qmon fin_sim lamda mu 
	$ns flush-trace 
	
	puts ""              
	puts "Fin de la simulacin.  Tiempo de simulacin:  $fin_sim s"      
	puts ""              
	puts ""    
	puts [format "Lambda = %5.5f" $lamda]
	puts [format "Mu = %5.5f" $mu]
	puts ""
	
	set parri [eval $qmon set parrivals_]      
	set barri [eval $qmon set barrivals_]
	set paten [eval $qmon set pdepartures_]      
	set baten [eval $qmon set bdepartures_]   
	puts ""
	
	# se calcula la utilizacin del sistema de dos formas
	# (i)   cociente entre lamda y mu   
	# (ii)  cociente entre tiempo que el servidor est ocupado y tiempo total de simulacin
	
	set ro1 [expr 1.0*$lamda/$mu]
	set busy [expr 1.0*$paten/$mu]
	set ro2 [expr 1.0*$busy/$fin_sim]
		
	puts [format "Utilizacin del sistema (lambda/mu) = %9.3f" $ro1]
	puts ""
	puts [format "Utilizacin del sistema (T_busy/T_sim) = %9.3f" $ro2]
	puts ""
	
	set pktsInt [$qmon get-pkts-integrator]
	set suma [expr [$pktsInt set sum_]]
	set avg_queue [expr $suma/$fin_sim]
	puts [format "N promedio de paquetes en la cola = %9.5f paquetes" $avg_queue]
	puts ""
	
	close $tf 
	close $tq
	exec xgraph cola.q -geometry 800x400 &
	# exec nam out.nam &    
	exit 0 
} 

proc record {} {
	global ns qmon tq       
	set now [$ns now]
	set time 0.01
	set tamcol [eval $qmon set pkts_]
	puts $tq "$now $tamcol "
	$ns at [expr $now+$time] "record"
}


proc sendpacket {} {
    global ns src InterArrivalTime pktSize 
    set time [$ns now]
    $ns at [expr $time + [$InterArrivalTime value]] "sendpacket"
    set bytes [expr round ([$pktSize value])]
    $src send $bytes
}

$ns at 0 "record"
$ns at 0.0001 "sendpacket"
$ns at $fin_sim "finish"

$ns run

