#	n1--------n2--------n3


set ns [new Simulator]
set f [open out.tr w]
$ns trace-all $f
set fin_sim 1000

set lamda 8.0
set tarribo [expr 1/$lamda] 
set cenlace 80000

set cenlace1 80000
set tampa 500
set mu [expr $cenlace/(8*$tampa)]

# crea archivos para registrar el tamao de la cola de ambos enlaces
set tq [open cola.q w]
set tq1 [open cola1.q w]

#
# Se genera la topologa de la red
#

# crea los nodos
set n1 [$ns node]
set n2 [$ns node]
set n3 [$ns node]


# crea los enlaces
$ns simplex-link $n1 $n2 $cenlace 0ms DropTail
$ns simplex-link $n2 $n3 $cenlace1 0ms DropTail


# crea un agente UDP y lo asigna al nodo n1
set fuente [new Agent/UDP]
$ns attach-agent $n1 $fuente

# crea un receptor y lo asigna a n3.
set sink [new Agent/Null] 
$ns attach-agent $n3 $sink

# conecta la fuente y el receptor

$ns connect $fuente $sink

# crea un generador de trfico Poisson y lo asigna al nodo UDP

set tgen [new Application/Traffic/Exponential]
$tgen attach-agent $fuente

# configura los parmetros del generador de trfico tal que el mismo 
# genera un paquete cada cierto tiempo t, donde t es una V.A. con
# distribucin exponencial de media 1/lamda .

$tgen set packetSize_ $tampa
$tgen set burst_time_ 0ms
$tgen set idle_time_ $tarribo 
$tgen set rate_ 100Mb

# NOTA: El generador de trafico arriba configurado es un generador exponencial de tipo on/off, donde 
# burst_time e idle_time son respectivamente los tiempos durante el cual la fuente est activa e inactiva,
# siendo dichos tiempos V.A. con distribucin exponencial. 

# Para generar un proceso Poisson de generacin de paquetes seteamos el valor de burst_time en 0 (el ns 
# asegura que en este caso se enviar un solo paquete) y la tasa de generacin en un valor grande (de tal 
# forma de que el tiempo entre paquetes quede determinado puramente por el idle_time).


# crea un monitor de cola para cada enlace
$ns trace-queue $n1 $n2
set qmon [$ns monitor-queue $n1 $n2 ""]
$ns trace-queue $n2 $n3
set qmon1 [$ns monitor-queue $n2 $n3 ""]



# se establece el tamao mximo de la cola en paquetes
$ns queue-limit $n1 $n2 200000
$ns queue-limit $n2 $n3 200000


# Procedimiento que se llama periodicamente (cada time seg) para registrar el tamao de la cola.


       
proc record {} {
	global ns qmon qmon1 tq tq1     
	set now [$ns now]
	set time 0.01
	set tamcol [eval $qmon set pkts_]
	set tamco2 [eval $qmon1 set pkts_]
	puts $tq "$now $tamcol"
	puts $tq1 "$now $tamco2"
	$ns at [expr $now+$time] "record"
}




# Procedimiento de finalizacin

proc finish {} {
      global ns qmon qmon1 fin_sim lamda mu cont tot_delay 
        $ns flush-trace	

exec clear 
puts ""              
puts "Fin de la simulacin.  Tiempo de simulacin:  $fin_sim s"      
puts ""              
puts "Parmetros del trfico de entrada al sistema"    
puts ""
puts [format "Lambda = %5.5f" $lamda]
puts [format "Mu = %5.5f" $mu]
puts ""


set pktsInt [$qmon get-pkts-integrator]
set suma [expr [$pktsInt set sum_]]
set avg_queue [expr $suma/$fin_sim]
puts [format "N promedio de paquetes en cola 1 = %9.5f paquetes" $avg_queue]
puts ""
puts ""
puts "Datos de la cola 2"
set pktsInt [$qmon1 get-pkts-integrator]
set suma [expr [$pktsInt set sum_]]
set avg_queue [expr $suma/$fin_sim]
puts [format "N promedio de paquetes en cola 2 = %9.5f paquetes" $avg_queue]
puts ""


exec xgraph cola.q cola1.q -geometry 800x400 &

exit 0

}

# se indican las operaciones

$ns at 0 "record"
$ns at 0.01 "$tgen start"
$ns at $fin_sim "$tgen stop"
$ns at [expr $fin_sim+0.01] "finish"

$ns run


