#    n1---------n2---------n3



set ns [new Simulator]

set tf [open out.tr w] 
$ns trace-all $tf

set lamda 50.0
set mu     80.0
set fin_sim 1000
set cenlace 100000
set cenlace1 100000


set n1 [$ns node]
set n2 [$ns node]
set n3 [$ns node]

# Como el tamao de los paquetes ser generado aleatoriamente, debemos redondear su valor
# a un entero. Para tener pequeos errores en el redondeo, tomaremos tamaos de paquete 
# grandes y, en consecuencia, gran ancho de banda para el enlace.

$ns simplex-link $n1 $n2 $cenlace 0ms DropTail
$ns simplex-link $n2 $n3 $cenlace1 0ms DropTail


# Limitamos el tamao de la cola

$ns queue-limit $n1 $n2 1000000
$ns queue-limit $n2 $n3 1000000


# Generamos aleatoriamente tiempos de arribo de paquetes y tamaos de paquetes de acuerdo
# a una distribucin exponencial.

set InterArrivalTime [new RandomVariable/Exponential]
$InterArrivalTime set avg_ [expr 1/$lamda]
set pktSize [new RandomVariable/Exponential]
$pktSize set avg_ [expr $cenlace/(8*$mu)]

set src [new Agent/UDP]
$ns attach-agent $n1 $src

set sink [new Agent/Null]
$ns attach-agent $n3 $sink
$ns connect $src $sink

# Creamos un monitor de cola para cada enlace 

set tq [open cola.q w]
set tq1 [open cola1.q w]

set qmon [$ns monitor-queue $n1 $n2 ""]
set qmon1 [$ns monitor-queue $n2 $n3 ""]


proc record {} {
	global ns qmon qmon1 tq tq1       
	set now [$ns now]
	set time 0.01
	set tamcol [eval $qmon set pkts_]
	set tamcol1 [eval $qmon1 set pkts_]
	puts $tq "$now $tamcol"
	puts $tq1 "$now $tamcol1"
	$ns at [expr $now+$time] "record"
}



proc finish {} {
	global ns tf tq tq1 qmon qmon1 fin_sim lamda mu
	$ns flush-trace 
	
	exec clear 
	puts ""              
	puts "Fin de la simulacin.  Tiempo de simulacin:  $fin_sim s"      
	puts ""              
	puts "Parmetros del trfico de entrada al sistema"    
	puts ""
	puts [format "Lambda = %5.5f" $lamda]
	puts [format "Mu = %5.5f" $mu]
	puts ""
		
	set pktsInt [$qmon get-pkts-integrator]
	set suma [expr [$pktsInt set sum_]]
	set avg_queue [expr $suma/$fin_sim]
	puts [format "N promedio de paquetes en cola 1 = %9.5f paquetes" $avg_queue]
	puts ""
	puts ""
	puts "Datos de la cola 2"
	puts ""
	set pktsInt1 [$qmon1 get-pkts-integrator]
	set suma1 [expr [$pktsInt1 set sum_]]
	set avg_queue1 [expr $suma1/$fin_sim]
	puts [format "N promedio de paquetes en cola 2 = %9.5f paquetes" $avg_queue1]
	puts ""
	
	close $tf 
	close $tq
	close $tq1
	exec xgraph cola.q cola1.q -geometry 800x400 &
	    
	exit 0 
} 




proc sendpacket {} {
    global ns src InterArrivalTime pktSize 
    set time [$ns now]
    $ns at [expr $time + [$InterArrivalTime value]] "sendpacket"
    set bytes [expr round ([$pktSize value])]
    $src send $bytes
}

$ns at 0 "record"
$ns at 0.0001 "sendpacket"
$ns at $fin_sim "finish"

$ns run

